import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

/**
 * A checker board.
 * 
 * @author David J. Barnes
 * @version 2008.08.06
 */
public class Board extends World
{
    // The size of each cell on the board.
    public static final int CELLSIZE = 60;
    public static final int MIN_X = 0, MAX_X = 7;
    public static final int MIN_Y = 0, MAX_Y = 7;
    // Which piece's turn it is next.
    private Class currentTurn = DarkMan.class;
    
    /**
     * Constructor for objects of class Board.
     * 
     */
    public Board()
    {        
        // Create a new world.
        super(MAX_X + 1, MAX_Y + 1, CELLSIZE);
        
        // Create the image for the board.  A 2x2 checker pattern.
        GreenfootImage background = new GreenfootImage(CELLSIZE * 2, CELLSIZE * 2);
        background.setColor(Color.WHITE);
        background.fillRect(0, 0, background.getWidth(), background.getHeight());
        background.setColor(Color.BLACK);
        background.fillRect(CELLSIZE, 0, CELLSIZE, CELLSIZE);
        background.fillRect(0, CELLSIZE, CELLSIZE, CELLSIZE);
        setBackground(background); 

        startNewGame();
    }
    
    /**
     * Populate the board with the starting pieces of the game.
     */
    public void startNewGame()
    {
        removeObjects(getObjects(Actor.class));
        
        for(int x = MIN_X; x <= MAX_X; x += 2) {
            addObject(new ThinkingMan(DarkMan.class), x + 1, MIN_Y);
            addObject(new ThinkingMan(DarkMan.class), x,     MIN_Y + 1);
            addObject(new ThinkingMan(DarkMan.class), x + 1, MIN_Y + 2);
            
            addObject(new DarkMan(ThinkingMan.class), x,     MAX_Y - 2);
            addObject(new DarkMan(ThinkingMan.class), x + 1, MAX_Y - 1);
            addObject(new DarkMan(ThinkingMan.class), x,     MAX_Y);
        }
        currentTurn = DarkMan.class;
    }
    
    /**
     * Find a list of actors with moves available and select one to be played.
     */
    public void act()
    {
        // Identify those with moves available.
        List<Actor> actors = getObjects(Man.class);
        List<Man> available = new LinkedList<Man>();
        for(Actor actor : actors) {
            Man man = (Man) actor;
            if(man.isMoveAvailable()) {
                available.add(man);
            }
        }
        
        // Pick a piece from those available and play it.
        if(available.size() > 0) {
            Man selected = selectMan(available);
            if(selected.play()) {
                // Ok.
            }
            else {
                throw new IllegalStateException(selected.toString());
            }
        }
        else {
            // It could be a stalemate or simply waiting for the player to move
            // the DarkMan pieces.
        }
    }
    
    /**
     * The class of actor whose turn it is at the moment.
     * @return Whose turn it is.
     */
    public Class getCurrentTurn()
    {
        return currentTurn;
    }
    
    /**
     * Switch whose turn it is
     */
    public void switchSides()
    {
        if(currentTurn.equals(DarkMan.class)) {
            currentTurn = ThinkingMan.class;
        }
        else {
            currentTurn = DarkMan.class;
        }
        if(getObjects(currentTurn).size() == 0) {
            // Game over.
            Greenfoot.stopSimulation();
        }
    }
    
    /**
     * Select the piece from the list with the longest move path.
     * @param available List of pieces that can move.
     */
    private Man selectMan(List<Man> available)
    {
        Man selected = available.get(0);
        int pathLength = selected.getMovePath().size();
        for(int index = 1; index < available.size(); index++) {
            List<Location> possiblePath = available.get(index).getMovePath();
            if(possiblePath.size() > pathLength) {
                selected = available.get(index);
                pathLength = possiblePath.size();
            }
            else if(possiblePath.size() == pathLength &&
                        Greenfoot.getRandomNumber(1) == 0) {
                // Perturb the natural iteration order.
                selected = available.get(index);
            }
            else {
                // A shorter move.
            }                    
        }
        return selected;
    }
        
}
