import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

/**
 * LightMan is a poor implementation of a checkers piece that makes
 * random single-square moves. It is simply provided as an illustration
 * of how a programmed implementation might plug into to functionality
 * inherited from the Man class.
 * 
 * @author David J. Barnes
 * @version 2008.08.05
 */
public class LightMan extends Man
{
    // This piece naturally moves down the board, until it becomes a king.
    private static final int NATURAL_DIRECTION = 1;
    
    /**
     * Indicate the color of piece to use, and the opponent type.
     */
    public LightMan(Class opponent)
    {
        super(Color.WHITE, opponent);
    }
    
    /**
     * Act - do whatever the LightMan wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        super.act();        
    }
    
    /**
     * Choose a path representing a move from the start location to
     * the end location, including any intermediate squares.
     * @return The chosen path, or null if none is chosen.
     */
    public List<Location> chooseMovePath()
    {
        // Build a path of the square to be moved through from
        // starting point to finishing point.
        // At the moment this just adds a single, randomly-generated,
        // finishing point.
        int x = getX(), y = getY();
        List<Location> path = new LinkedList<Location>();
        path.add(new Location(getX(), getY()));
        path.add(randomMove());
        
        // If it is legal, make the selected move available.
        if(isLegal(path)) {
            return path;        
        }
        else {
            return null;
        }
    }
    
    /**
     * Generate a single-square movement from the current location.
     * If the piece is a King then backward movement is possible.
     * Ensure that the move is diagonal and not off the sides of the board.
     */
    private Location randomMove()
    {
        int x = getX(), y = getY();
        
        // Generate an x-offset of -1 or +1.
        int dx = Greenfoot.getRandomNumber(2);
        if(dx == 0) {
            dx = -1;
        }
        // Avoid moving off the edge of the board.
        if(x + dx < Board.MIN_X || x + dx > Board.MAX_X) {
            dx = -dx;
        }
        
        // Generate a y-offset. If the piece is a King it
        // can move backwards.
        int dy = NATURAL_DIRECTION;
        if(isKing() && Greenfoot.getRandomNumber(2) == 0) {
            dy = -NATURAL_DIRECTION;
        }        
        // Avoid moving off the edge of the board.
        if(y + dy < Board.MIN_Y || y + dy > Board.MAX_Y) {
            dy = -dy;
        }
        return new Location(x + dx, y + dy);           
    }
}
